# Yet another release downloader (YARD)

[![GitHub release](https://img.shields.io/github/release/ArKuznetsov/yard.svg?style=flat-square)](https://github.com/ArKuznetsov/yard/releases)
[![GitHub license](https://img.shields.io/github/license/ArKuznetsov/yard.svg?style=flat-square)](https://github.com/ArKuznetsov/yard/blob/develop/LICENSE)
[![Build Status](https://travis-ci.org/arkuznetsov/yard.svg?branch=develop)](https://travis-ci.org/arkuznetsov/yard)
[![Quality Gate](https://oskk-sonar.1solution.ru/api/badges/gate?key=yard)](https://oskk-sonar.1solution.ru/dashboard/index/yard)
[![Coverage](https://oskk-sonar.1solution.ru/api/badges/measure?key=yard&metric=coverage)](https://oskk-sonar.1solution.ru/dashboard/index/yard)
[![Tech debt](https://oskk-sonar.1solution.ru/api/badges/measure?key=yard&metric=sqale_debt_ratio)](https://oskk-sonar.1solution.ru/dashboard/index/yard)

Приложение oscript для загрузки и обработки релизов конфигураций 1С.

Требуются следующие библиотеки и инструменты:
- [logos](https://github.com/oscript-library/logos)
- [asserts](https://github.com/oscript-library/asserts)
- [cli](https://github.com/oscript-library/cli)
- [v8runner](https://github.com/oscript-library/v8runner)
- [gitrunner](https://github.com/oscript-library/gitrunner)
- [semver](https://github.com/oscript-library/semver)
- [tempfiles](https://github.com/oscript-library/tempfiles)
- [fs](https://github.com/oscript-library/fs)
- [7-zip](http://www.7-zip.org/)

## Общие сведения

- выполняет загрузку приложений с [сайта релизов 1С](https://releases.1c.ru)

- для релизов конфигураций доступны следующие возможности

    - загрузка указанных версий конфигурации
    - распаковка архива конфигурации (требуется 7-zip)
    - распаковка файлов шаблона конфигурации в указанный каталог (из формата EFD)
    - последовательная сборка CF-фалов конфигурации из CFU-файлов обновлений (требуется начальный CF)
    - последовательная выгрузка версий конфигурации в формате XML в указанный репозитарий GIT

- Реализованы 2 варианта работы:
    - запуск отдельных команд для каждого этапа обработки релизов конфигураций
    - экспериментальный пакетный режим с конвейерной обработкой данных (pipeline), т.е. каждая порция данных, полученная в текущей обработке передается в указанный(-ные) в настройках обработчик(-и) для последующе обработки
- настройки пакетного режима описываются в формате JSON (см. [Файл настроек](#jsonsettings))

## Команды

- **process** (p) - выполняет обработку данных настройкам из файла (.json)
  - _--work-dir_ - путь к рабочему каталогу (по умолчанию: текущий каталог)
  - _<Путь>_ - путь к файлу настроек (по умолчанию ./yardsettings.json)

#### Пример:
```bat
yard process --work-dir "c:\tmp" ".\examples\releases2templates.json"
```
см. [Файл настроек](#jsonsettings)

- **releases** (r) - работа с релизами с сайта 1С
  - _--user_ - пользователь портала 1С
  - _--pwd_ - пароль пользователя портала 1С

  - **list** (l) - получение списка приложений и версий с портала 1С
    - _--app-filter_ - фильтр приложений по имени (регулярное выражение)
    - _--version-filter_ - фильтр версий по номеру (регулярное выражение)
    - _--version-start-date_ - фильтр по начальной дате версии (включая)
    - _--version-end-date_ - фильтр по последней дате версии (включая)
    - _--output-file_ - путь к фалу (json) для сохранения списка приложений и версий

#### Пример:
```bat
yard releases --user "user" --pwd "pwd" list --app-filter "Библиотека стандартных подсистем.*3\.1" --version-filter 3\.1.* --version-start-date "01.01.2019" --output-file ".\tmp\releases.json"
```

  - **get** (g) - загрузка указанных версий, указанных приложений с портала 1С
    - _--app-filter_ - фильтр приложений по имени (регулярное выражение)
    - _--version-filter_ - фильтр версий по номеру (регулярное выражение)
    - _--version-start-date_ - фильтр по начальной дате версии (включая)
    - _--version-end-date_ - фильтр по последней дате версии (включая)
    - _--path_ - каталог для загрузки релизов 1С
    - _--extract_ - флаг распаковки загруженного архива
    - _--extract-path_ - каталог для распаковки загруженного архива
    - _--extract-files_ - список файлов для распаковки из архива дистрибутива, разделенный "|"
    - _--delete_ - флаг удаления загруженного архива после распаковки

#### Пример:
```bat
yard releases --user "user" --pwd "pwd" get --app-filter "Библиотека стандартных подсистем.*3\.1" --version-filter 3\.1.* --version-start-date "01.01.2019" --path ".\tmp\distr" --extract --extract-path ".\tmp\tmplts"
```

- **unpack** (u) - распаковка шаблона конфигурации 1С из EFD-файла дистрибутива конфигурации 1С
  - _--path_ - путь к дистрибутиву конфигурации 1С
  - _--extract-path_ - каталог для распаковки загруженного архива
  - _--extract-files_ - список файлов для распаковки из архива дистрибутива, разделенный "|"
  - _--app-name_ - имя конфигурации 1С (если не указано, значение будет считано из файла 1cv8.mft дистрибутива)
  - _--app-id_ - идентификатор конфигурации 1С (если не указано, значение будет считано из файла 1cv8.mft дистрибутива)
  - _--app-version_ - версия конфигурации 1С (если не указано, значение будет считано из файла 1cv8.mft дистрибутива)

#### Пример:
```bat
yard unpack --app-name "Библиотека стандартных подсистем, редакция 3.1" --app-id "SSL31" --app-version "3.1.2.332" --path ".\tmp\distr\SSL31\3.1.2.332" --extract-path ".\tmp\tmplts"
```

- **build-cf** (b) - выполняет обновление конфигурации (CF) файлом обновления (CFU) и помещает результат в новый файл CF
  - _--cf-path_ - путь к файлу конфигурации (CF) предыдущей версии
  - _--cfu-path_ - путь к файлу обновления (CFU) новой версии
  - _--ibconnection_ - строка подключения к служебной базе 1С для выполнения обновления (если не указана, будет использована временная ИБ)

#### Пример:
```bat
yard build-cf --cf-path ".\tmp\tmplts\1c\AccountingCorp\3_0_64_54\1cv8.cf" --cfu-path ".\tmp\tmplts\1c\AccountingCorp\3_0_66_53\1cv8.cfu" --ibconnection "/SMyServer\TMP_BASE"
```

- **build-all** (ba) - для всех файлов обновления (CFU) в каталоге ищет доступные для обновления файлы(CF) и выполняет обновление
  - _--path_ - путь к каталогу содержащему версии конфигурации для формирования CF-файлов
  - _--ibconnection_ - строка подключения к служебной базе 1С для выполнения обновления (если не указана, будет использована временная ИБ)

#### Пример:
```bat
yard build-all --path ".\tmp\tmplts\1c\AccountingCorp" --ibconnection "/SMyServer\TMP_BASE"
```

- **upload-cf** (ucf) - выполняет выгрузку конфигурации 1С из файла конфигурации (CF) в репозитарий git
  - _--cf-path_ - путь к файлу конфигурации (CF) для выгрузки
  - _--git-path_ - путь к репозитарию git
  - _--git-branch_ - имя ветки git в которую будет выполняться выгрузка (по умолчанию: base1c)
  - _--git-author_ - имя автора коммита в git (по умолчанию: 1c)
  - _--git-author-email_ - почта автора коммита в git (по умолчанию: 1c@1c.ru)
  - _--git-commit-date_ - дата коммита в git в формате POSIX (по умолчанию: <ТекущаяУниверсальнаяДата>)
  - _--git-commit-message_ - сообщение коммита в git
  - _--ibconnection_ - строка подключения к служебной базе 1С для выполнения обновления (если не указана, будет использована временная ИБ)

#### Пример:
```bat
yard upload-cf --cf-path ".\tmp\tmplts\SSL31\3.1.1.91\1cv8.cf" --git-path ".\tmp\repo\SSL31" --git-commit-message "Тест выгрузки БСП 3.1.1.91" --ibconnection "/SMyServer\TMP_BASE"
```

- **upload-all** (ua) - выполняет выгрузку всех версий конфигурации 1С в каталоге в репозитарий git
  - _--path_ - путь к каталогу содержащему версии конфигурации для выгрузки в git
  - _--cf-name - имя файла конфигурации (по умолчанию: 1cv8.cf)
  - _--git-path_ - путь к репозитарию git
  - _--git-author_ - имя автора коммита в git (по умолчанию: 1c)
  - _--git-author-email_ - почта автора коммита в git (по умолчанию: 1c@1c.ru)
  - _--ibconnection_ - строка подключения к служебной базе 1С для выполнения обновления (если не указана, будет использована временная ИБ)

#### Пример:
```bat
yard upload-all --path ".\tmp\tmplts\SSL31\" --cfname "1cv8.cf" --git-path ".\tmp\repo\SSL31" --ibconnection "/SMyServer\TMP_BASE"
```

## Управляющие обработки

### МенеджерОбработкиДанных.os

Управляющая обработка-менеджер, читает настройки, запускает и управляет обработкой данных.

### yard.os

Обработка для интерактивного выполнения команд обработки релизов.

## <a id="api"></a> Стандартный программный интерфейс обработки

**Функция ОписаниеПараметров()** - возвращает структуру с описанием параметров обработки

**Функция МенеджерОбработкиДанных()** - возвращает ссылку на вызывающую/управляющую обработку - менеджер обработки данных

**Процедура УстановитьМенеджерОбработкиДанных(Знач НовыйМенеджерОбработкиДанных)** - устанавливает ссылку на вызывающую/управляющую обработку - менеджер обработки данных

**Функция Идентификатор()** - возвращает идентификатор обработки, установленный при инициализации в менеджере обработки данных

**Процедура УстановитьИдентификатор(Знач НовыйИдентификатор)** - устанавливает идентификатор обработки, вызывается при инициализации в менеджере обработки данных

**Функция ПараметрыОбработкиДанных()** - возвращает значения параметров обработки данных

**Процедура УстановитьПараметрыОбработкиДанных(Знач НовыеПараметры)** - устанавливает значения параметров обработки данных

**Функция ПараметрОбработкиДанных(Знач ИмяПараметра)** - возвращает значение указанного параметра обработки данных

**Процедура УстановитьПараметрОбработкиДанных(Знач ИмяПараметра, Знач Значение)** - устанавливает значение указанного параметра обработки

**Процедура УстановитьДанные(Знач ВходящиеДанные)** - устанавливает данные для обработки

**Процедура ОбработатьДанные()** - выполняет обработку данных

**Функция РезультатОбработки()** - возвращает результаты обработки данных

**Процедура ЗавершениеОбработкиДанных()** - выполняет действия при окончании обработки данных и оповещает обработку-менеджер о завершении обработки данных

## Обработчики данных

### СписокРелизов1С.os

Получает с сайта 1С список релизов и выводит в консоль или в указанный файл.

#### Параметры

- **ИмяПользователя** - Имя пользователя сайта релизов 1С
- **ПарольПользователя** - Пароль пользователя сайта релизов 1С
- **ФильтрПриложений** - Фильтр имен приложений
- **ФильтрВерсий** - Фильтр номеров версий
- **ФильтрВерсийНачинаяСДаты** - Фильтр по начальной дате версии (включая)
- **ФильтрВерсийДоДаты** - Фильтр по последней дате версии (включая)
- **ПутьКФайлуДляСохранения** - путь к фалу (json) для сохранения списка приложений и версий

### ЗагрузчикРелизов1С.os

Выполняет загрузку указанных приложений и версий с сайта 1С и, при необходимости, распаковывает файлы шаблона конфигурации 1С.

#### Параметры

- **ИмяПользователя** - Имя пользователя сайта релизов 1С
- **ПарольПользователя** - Пароль пользователя сайта релизов 1С
- **ФильтрПриложений** - Фильтр имен приложений
- **ФильтрВерсий** - Фильтр номеров версий
- **ФильтрВерсийНачинаяСДаты** - Фильтр по начальной дате версии (включая)
- **ФильтрВерсийДоДаты** - Фильтр по последней дате версии (включая)
- **КаталогДляСохранения** - каталог для загрузки релизов 1С
- **РаспаковыватьEFD** - Истина - если загруженный архив содержит упакованный шаблонконфигурации (содержит файл 1cv8.efd), то он будет распакован
- **КаталогДляРаспаковкиEFD** - каталог для распаковки шаблона конфигурации
- **ФайлыДляРаспаковкиEFD** - список файлов для распаковки из архива EFD дистрибутива конфигурации, если не указан, то распаковываются все файлы
- **УдалитьПослеРаспаковкиEFD** - Истина - после рапаковки загруженный архив будет удален

### РаспаковщикРелизов1С.os

Выполняет распаковку файлов шаблона конфигурации 1С из EFD-файла дистрибутива.

#### Параметры

- **Приложение_Имя** - имя конфигурации 1С (если не указано, значение будет считано из файла 1cv8.mft дистрибутива)
- **Приложение_Ид** - идентификатор конфигурации 1С (если не указано, значение будет считано из файла 1cv8.mft дистрибутива)
- **Приложение_Версия** - версия конфигурации 1С (если не указано, значение будет считано из файла 1cv8.mft дистрибутива)
- **ПутьКДистрибутиву** - путь к дистрибутиву конфигурации 1С
- **КаталогДляРаспаковкиEFD** - каталог для распаковки шаблона конфигурации
- **ФайлыДляРаспаковкиEFD** - список файлов для распаковки из архива EFD дистрибутива конфигурации, если не указан, то распаковываются все файлы

### СборщикКонфигураций1С.os

Выполняет обновление конфигурации (CF) файлом обновления (CFU) и помещает результат в новый файл CF.

#### Параметры

- **ПутьККонфигурации** - путь к файлу конфигурации (CF) предыдущей версии
- **ПутьКОбновлению** - путь к файлу обновления (CFU) новой версии
- **База_СтрокаСоединения** - строка подключения к служебной базе 1С для выполнения обновления (если не указана, будет использована временная ИБ)

### СборщикКонфигураций1СВКаталоге.os

Для всех файлов обновления (CFU) в каталоге ищет доступные для обновления файлы(CF) и выполняет обновление.

#### Параметры

- **ПутьККаталогуКонфигураций** - путь к каталогу содержащему версии конфигурации для формирования CF-файлов
- **База_СтрокаСоединения** - строка подключения к служебной базе 1С для выполнения обновления (если не указана, будет использована временная ИБ)

### ВыгрузкаКонфигурацииВГит.os

Выполняет выгрузку конфигурации 1С из файла конфигурации (CF) в репозитарий git.

#### Параметры

- **ПутьККонфигурации** - путь к файлу конфигурации (CF) для выгрузки
- **РепозитарийГит** - путь к репозитарию git
- **ИмяВеткиГит** - имя ветки git в которую будет выполняться выгрузка (по умолчанию: base1c)
- **ИмяАвтора** - имя автора коммита в git (по умолчанию: 1c)
- **ПочтаАвтора** - почта автора коммита в git (по умолчанию: 1c@1c.ru)
- **ДатаКоммита** - дата коммита в git в формате POSIX
- **СообщениеКоммита** - сообщение коммита в git
- **База_СтрокаСоединения** - строка подключения к служебной базе 1С для выполнения обновления (если не указана, будет использована временная ИБ)

### ВыгрузкаКаталогаКонфигурацииВГит.os

Выполняет выгрузку конфигурации 1С из файла конфигурации (CF) в репозитарий git.

#### Параметры

- **ПутьККонфигурации** - путь к каталогу содержащему версии конфигурации для выгрузки в git
- **ИмяФайлаКонфигурации** - имя файла конфигурации (по умолчанию: "1Cv8.cf")
- **РепозитарийГит** - путь к репозитарию git
- **ИмяАвтора** - имя автора коммита в git (по умолчанию: 1c)
- **ПочтаАвтора** - почта автора коммита в git (по умолчанию: 1c@1c.ru)
- **База_СтрокаСоединения** - строка подключения к служебной базе 1С для выполнения обновления (если не указана, будет использована временная ИБ)

## <a id="jsonsettings"></a> Файл настроек (JSON)

### Структура файла настроек

Файл настроек описывает последовательность вызова обработчиков для обработки данных. обработчик = обработка 1С реализующая вышеуказанный [API](#api).

```txt
    |-> Описание обработчика
    |      |- ИдОбработчика     - строковый идентификатор обработчика (необязательный)
    |      |- ИмяОбработки      - имя класса обработчика
    |      |- ПутьКОбработке    - путь к oscript-файлу класса обработчика
    |      |- Параметры         - структура параметров обработки
    |      |      |- <просто параметр>                 - параметр простого типа
    |      |      |- <параметр из данных обработчика>  - параметр вычисляемый обработчиком
    |      |      |      |- ИдОбработчика              - идентификатор обработчика из которого будет получен параметр
    |      |      |      |- ФункцияПолученияЗначения   - имя функции получения значения параметра (по умолчанию: "ПолучитьРезультат")
    |      |- Обработчики      - массив обработчиков данных полученных на текущем уровне
    |             |- <Описание обработчика>*           - структура, аналогичная данной
    |____________________|
```

### Доступные подстановки

- **$settingsDir** - каталог файла настроек
- **$yardDir** - каталог запуска скрипта
- **$workDir** - указанный при запуске рабочий каталог (по умолчанию: каталог файла настроек)

### Пример файла настроек

Пример файла настроек чтения словарей журнала регистрации

```json
{
    // имя класса обработчика получения списка релизов
    "ИмяОбработки":"СписокРелизов1С",
    "Параметры":{
        "ИмяПользователя" : "user",
        "ПарольПользователя" : "password",
        // Регулярки для фильтра приложений/конфигураций по имени
        "ФильтрПриложений" : ["Библиотека стандартных подсистем.*3\\.1"],
        // Регулярки для фильтра версий по номеру
        "ФильтрВерсий" : ["3\\.1\\.2.*"],
        "ФильтрВерсийНачинаяСДаты" : "2020-01-01T00:00:00",
    },
    "Обработчики":[
        {
            // имя класса обработчика загрузки релизов
            "ИмяОбработки":"ЗагрузчикРелизов1С",
            "ИдОбработчика":"ЗагрузчикБСП",
            "Параметры":{
                "ИмяПользователя" : "user",
                "ПарольПользователя" : "password",
                "КаталогДляСохранения" : "$workDir\\tmp\\distr",
                // флаг распаковки шаблона
                "РаспаковыватьEFD" : true,
                // путь распаковки шаблона конфигурации
                "КаталогДляРаспаковкиEFD" : "$workDir\\tmp\\tmplts"
            }
        }
    ]
}
```

Больше примеров расположены в каталоге [examples](./examples).
